"""
Created by adam on 3/16/20
"""
__author__ = 'adam'




if __name__ == '__main__':
    pass


class SkaaReviewGroup:
    """Activities which are part of a skaa"""
    pass


class DiscussionGroup:
    """Activities which are part of a discussion-review sequence"""
    pass

class ReviewType:
    """Activities which are reviews. Differentiated
    because may have multiple choice items and other differences
    """
    pass